<?php
declare(strict_types=1);

namespace FiloBlu\Refilo\Remote\Entity\Provider;

use FiloBlu\Refilo\Remote\Entity\EntityProviderInterface;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Model\Product;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;
use Magento\Framework\App\State;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Api\Data\WebsiteInterface;

/**
 * Class RelatedProvider
 * @package FiloBlu\Refilo\Remote\Entity\Provider
 */
class RelatedProvider extends BaseProvider
{
    /**
     * @var array
     */
    protected $ids = null;

    /**
     * @var \FiloBlu\Refilo\Model\UpsellFactory
     */
    private $upsellFactory;
    /**
     * @var CollectionFactory
     */
    private $productCollectionFactory;
    /**
     * @var State
     */
    private $state;
    /**
     * @var \FiloBlu\Refilo\Model\RelatedFactory
     */
    private $relatedFactory;
    /**
     * @var \Magento\TargetRule\Helper\Data
     */
    private $targetHelper;

    /**
     * RelatedProvider constructor.
     * @param State $state
     * @param CollectionFactory $productCollectionFactory
     * @param \FiloBlu\Refilo\Model\UpsellFactory $upsellFactory
     * @param \FiloBlu\Refilo\Model\RelatedFactory $relatedFactory
     * @param \Magento\TargetRule\Helper\Data $targetHelper
     */
    public function __construct(
        State $state,
        CollectionFactory $productCollectionFactory,
        \FiloBlu\Refilo\Model\UpsellFactory $upsellFactory,
        \FiloBlu\Refilo\Model\RelatedFactory $relatedFactory,
        \Magento\TargetRule\Helper\Data $targetHelper
    )
    {
        $this->upsellFactory = $upsellFactory;
        $this->productCollectionFactory = $productCollectionFactory;
        $this->state = $state;
        $this->relatedFactory = $relatedFactory;
        $this->targetHelper = $targetHelper;
    }

    /**
     * @param array $ids
     * @return EntityProviderInterface
     */
    public function withIds(array $ids): EntityProviderInterface
    {
        $this->ids = $ids;
        return $this;
    }

    /**
     * @param StoreInterface $store
     * @return EntityProviderInterface
     */
    public function forStore(StoreInterface $store): EntityProviderInterface
    {
        return $this;
    }

    /**
     * @param WebsiteInterface $website
     * @return EntityProviderInterface
     */
    public function forWebsite(WebsiteInterface $website): EntityProviderInterface
    {
        return $this;
    }

    /**
     *
     */
    public function rewind()
    {
        $this->items = [];
        $products = $this->productCollectionFactory->create()->getItems();

        $upsellBehaviour = (int)$this->targetHelper->getShowProducts(\Magento\TargetRule\Model\Rule::UP_SELLS);
        $relatedBehaviour = (int)$this->targetHelper->getShowProducts(\Magento\TargetRule\Model\Rule::RELATED_PRODUCTS);
        $crosssellBehaviour = (int)$this->targetHelper->getShowProducts(\Magento\TargetRule\Model\Rule::CROSS_SELLS);

        /** @var ProductInterface|Product $product */
        foreach ($products as $product) {

            $upsellIds = [];
            $crosssellIds = [];
            $relatedIds = [];

            $entityId = (int)$product->getId();

            $this->items[$entityId] = new \FiloBlu\Refilo\Remote\Entity\ProductRelated([
                'id'       => $entityId,
                'upsell'   => $this->getUpSellProductIds($product, $upsellBehaviour),
                'related'  => $this->getRelatedProductsIds($product, $relatedBehaviour),
                'crossell' => $this->getCrossSellProductIds($product, $crosssellBehaviour)
            ]);

        }

        if (!$this->hasReadHandler()) {
            return;
        }

        // TODO: Handle pagination
        $this->getReadHandler()->onRead($this->items, count($this->items), $this->getReadHandlerArguments());
    }


    /**
     * @param Product $product
     * @param int $behaviour
     * @return array
     */
    public function getUpSellProductIds($product, $behaviour)
    {
        $ids = [];

        switch ($behaviour) {
            case \Magento\TargetRule\Model\Rule::BOTH_SELECTED_AND_RULE_BASED:
                $ids = array_merge($product->getUpSellProductIds(), $this->upsellFactory->create()->setProduct($product)->getAllIds());
                break;
            case \Magento\TargetRule\Model\Rule::SELECTED_ONLY:
                $ids = $product->getUpSellProductIds();
                break;
            case \Magento\TargetRule\Model\Rule::RULE_BASED_ONLY:
                $ids = $this->upsellFactory->create()->setProduct($product)->getAllIds();
                break;
        }

        return array_map('intval', $ids);
    }

    /**
     * @param Product $product
     * @param int $behaviour
     * @return array
     */
    public function getRelatedProductsIds($product, $behaviour)
    {
        $ids = [];

        switch ($behaviour) {
            case \Magento\TargetRule\Model\Rule::BOTH_SELECTED_AND_RULE_BASED:
                $ids = array_merge($product->getRelatedProductIds(), $this->relatedFactory->create()->setProduct($product)->getAllIds());
                break;
            case \Magento\TargetRule\Model\Rule::SELECTED_ONLY:
                $ids = $product->getRelatedProductIds();
                break;
            case \Magento\TargetRule\Model\Rule::RULE_BASED_ONLY:
                $ids = $this->relatedFactory->create()->setProduct($product)->getAllIds();
                break;
        }

        return array_map('intval', $ids);
    }


    /**
     * @param Product $product
     * @param $behaviour
     * @return array
     */
    public function getCrossSellProductIds($product, $behaviour)
    {
        $ids = [];

        switch ($behaviour) {
            case \Magento\TargetRule\Model\Rule::BOTH_SELECTED_AND_RULE_BASED:
                $ids = array_merge($product->getRelatedProductIds(), []);
                break;
            case \Magento\TargetRule\Model\Rule::SELECTED_ONLY:
                $ids = $product->getCrossSellProductIds();
                break;
            case \Magento\TargetRule\Model\Rule::RULE_BASED_ONLY:
                $ids = [];
                break;
        }

        return array_map('intval', $ids);
    }
}
